@@ -2,17 +2,20 @@
 
 use crate::reexport::*;
 use crate::utils::{
-    in_macro, last_line_of_span, match_def_path, opt_def_id, paths, snippet_opt, span_lint, span_lint_and_then,
-    without_block_comments,
+    in_macro, last_line_of_span, match_def_path, opt_def_id, paths, snippet_opt, span_lint, span_lint_and_sugg,
+    span_lint_and_then, without_block_comments,
 };
-use crate::rustc::hir::*;
-use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
-use crate::rustc::{declare_tool_lint, lint_array};
 use if_chain::if_chain;
-use crate::rustc::ty::{self, TyCtxt};
+use rustc::hir::*;
+use rustc::lint::{
+    CheckLintNameResult, EarlyContext, EarlyLintPass, LateContext, LateLintPass, LintArray, LintContext, LintPass,
+};
+use rustc::ty::{self, TyCtxt};
+use rustc::{declare_tool_lint, lint_array};
+use rustc_errors::Applicability;
 use semver::Version;
-use crate::syntax::ast::{AttrStyle, Attribute, Lit, LitKind, MetaItemKind, NestedMetaItem, NestedMetaItemKind};
-use crate::syntax::source_map::Span;
+use syntax::ast::{AttrStyle, Attribute, Lit, LitKind, MetaItemKind, NestedMetaItem, NestedMetaItemKind};
+use syntax::source_map::Span;
 
 /// **What it does:** Checks for items annotated with `#[inline(always)]`,
 /// unless the annotated function is empty or simply panics.
@@ -127,6 +130,62 @@ declare_clippy_lint! {
     "empty line after outer attribute"
 }
 
+/// **What it does:** Checks for `allow`/`warn`/`deny`/`forbid` attributes with scoped clippy
+/// lints and if those lints exist in clippy. If there is a uppercase letter in the lint name
+/// (not the tool name) and a lowercase version of this lint exists, it will suggest to lowercase
+/// the lint name.
+///
+/// **Why is this bad?** A lint attribute with a mistyped lint name won't have an effect.
+///
+/// **Known problems:** None.
+///
+/// **Example:**
+/// Bad:
+/// ```rust
+/// #![warn(if_not_els)]
+/// #![deny(clippy::All)]
+/// ```
+///
+/// Good:
+/// ```rust
+/// #![warn(if_not_else)]
+/// #![deny(clippy::all)]
+/// ```
+declare_clippy_lint! {
+    pub UNKNOWN_CLIPPY_LINTS,
+    style,
+    "unknown_lints for scoped Clippy lints"
+}
+
+/// **What it does:** Checks for `#[cfg_attr(rustfmt, rustfmt_skip)]` and suggests to replace it
+/// with `#[rustfmt::skip]`.
+///
+/// **Why is this bad?** Since tool_attributes ([rust-lang/rust#44690](https://github.com/rust-lang/rust/issues/44690))
+/// are stable now, they should be used instead of the old `cfg_attr(rustfmt)` attributes.
+///
+/// **Known problems:** This lint doesn't detect crate level inner attributes, because they get
+/// processed before the PreExpansionPass lints get executed. See
+/// [#3123](https://github.com/rust-lang/rust-clippy/pull/3123#issuecomment-422321765)
+///
+/// **Example:**
+///
+/// Bad:
+/// ```rust
+/// #[cfg_attr(rustfmt, rustfmt_skip)]
+/// fn main() { }
+/// ```
+///
+/// Good:
+/// ```rust
+/// #[rustfmt::skip]
+/// fn main() { }
+/// ```
+declare_clippy_lint! {
+    pub DEPRECATED_CFG_ATTR,
+    complexity,
+    "usage of `cfg_attr(rustfmt)` instead of `tool_attributes`"
+}
+
 #[derive(Copy, Clone)]
 pub struct AttrPass;
 
@@ -136,21 +195,32 @@ impl LintPass for AttrPass {
             INLINE_ALWAYS,
             DEPRECATED_SEMVER,
             USELESS_ATTRIBUTE,
-            EMPTY_LINE_AFTER_OUTER_ATTR
+            EMPTY_LINE_AFTER_OUTER_ATTR,
+            UNKNOWN_CLIPPY_LINTS,
         )
     }
+
+    fn name(&self) -> &'static str {
+        "Attributes"
+    }
 }
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for AttrPass {
     fn check_attribute(&mut self, cx: &LateContext<'a, 'tcx>, attr: &'tcx Attribute) {
-        if let Some(ref items) = attr.meta_item_list() {
+        if let Some(items) = &attr.meta_item_list() {
+            match &*attr.name().as_str() {
+                "allow" | "warn" | "deny" | "forbid" => {
+                    check_clippy_lint_names(cx, items);
+                },
+                _ => {},
+            }
             if items.is_empty() || attr.name() != "deprecated" {
                 return;
             }
             for item in items {
                 if_chain! {
-                    if let NestedMetaItemKind::MetaItem(ref mi) = item.node;
-                    if let MetaItemKind::NameValue(ref lit) = mi.node;
+                    if let NestedMetaItemKind::MetaItem(mi) = &item.node;
+                    if let MetaItemKind::NameValue(lit) = &mi.node;
                     if mi.name() == "since";
                     then {
                         check_semver(cx, item.span, lit);
@@ -162,33 +232,33 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for AttrPass {
 
     fn check_item(&mut self, cx: &LateContext<'a, 'tcx>, item: &'tcx Item) {
         if is_relevant_item(cx.tcx, item) {
-            check_attrs(cx, item.span, item.name, &item.attrs)
+            check_attrs(cx, item.span, item.ident.name, &item.attrs)
         }
         match item.node {
             ItemKind::ExternCrate(..) | ItemKind::Use(..) => {
                 let skip_unused_imports = item.attrs.iter().any(|attr| attr.name() == "macro_use");
 
                 for attr in &item.attrs {
-                    if let Some(ref lint_list) = attr.meta_item_list() {
+                    if let Some(lint_list) = &attr.meta_item_list() {
                         match &*attr.name().as_str() {
                             "allow" | "warn" | "deny" | "forbid" => {
                                 // whitelist `unused_imports` and `deprecated` for `use` items
                                 // and `unused_imports` for `extern crate` items with `macro_use`
                                 for lint in lint_list {
                                     match item.node {
-                                        ItemKind::Use(..) => if is_word(lint, "unused_imports")
-                                                                || is_word(lint, "deprecated") {
-                                                return
+                                        ItemKind::Use(..) => {
+                                            if is_word(lint, "unused_imports") || is_word(lint, "deprecated") {
+                                                return;
+                                            }
                                         },
                                         ItemKind::ExternCrate(..) => {
-                                            if is_word(lint, "unused_imports")
-                                                && skip_unused_imports {
-                                                    return
+                                            if is_word(lint, "unused_imports") && skip_unused_imports {
+                                                return;
                                             }
                                             if is_word(lint, "unused_extern_crates") {
-                                                return
+                                                return;
                                             }
-                                        }
+                                        },
                                         _ => {},
                                     }
                                 }
@@ -203,7 +273,12 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for AttrPass {
                                             "useless lint attribute",
                                             |db| {
                                                 sugg = sugg.replacen("#[", "#![", 1);
-                                                db.span_suggestion(line_span, "if you just forgot a `!`, use", sugg);
+                                                db.span_suggestion(
+                                                    line_span,
+                                                    "if you just forgot a `!`, use",
+                                                    sugg,
+                                                    Applicability::MachineApplicable,
+                                                );
                                             },
                                         );
                                     }
@@ -231,9 +306,55 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for AttrPass {
     }
 }
 
+#[allow(clippy::single_match_else)]
+fn check_clippy_lint_names(cx: &LateContext<'_, '_>, items: &[NestedMetaItem]) {
+    let lint_store = cx.lints();
+    for lint in items {
+        if_chain! {
+            if let Some(word) = lint.word();
+            if let Some(tool_name) = word.is_scoped();
+            if tool_name.as_str() == "clippy";
+            let name = word.name();
+            if let CheckLintNameResult::Tool(Err((None, _))) = lint_store.check_lint_name(
+                &name.as_str(),
+                Some(tool_name.as_str()),
+            );
+            then {
+                span_lint_and_then(
+                    cx,
+                    UNKNOWN_CLIPPY_LINTS,
+                    lint.span,
+                    &format!("unknown clippy lint: clippy::{}", name),
+                    |db| {
+                        if name.as_str().chars().any(char::is_uppercase) {
+                            let name_lower = name.as_str().to_lowercase();
+                            match lint_store.check_lint_name(
+                                &name_lower,
+                                Some(tool_name.as_str())
+                            ) {
+                                // FIXME: can we suggest similar lint names here?
+                                // https://github.com/rust-lang/rust/pull/56992
+                                CheckLintNameResult::NoLint(None) => (),
+                                _ => {
+                                    db.span_suggestion(
+                                        lint.span,
+                                        "lowercase the lint name",
+                                        name_lower,
+                                        Applicability::MaybeIncorrect,
+                                    );
+                                }
+                            }
+                        }
+                    }
+                );
+            }
+        };
+    }
+}
+
 fn is_relevant_item(tcx: TyCtxt<'_, '_, '_>, item: &Item) -> bool {
     if let ItemKind::Fn(_, _, _, eid) = item.node {
-        is_relevant_expr(tcx, tcx.body_tables(eid), &tcx.hir.body(eid).value)
+        is_relevant_expr(tcx, tcx.body_tables(eid), &tcx.hir().body(eid).value)
     } else {
         true
     }
@@ -241,7 +362,7 @@ fn is_relevant_item(tcx: TyCtxt<'_, '_, '_>, item: &Item) -> bool {
 
 fn is_relevant_impl(tcx: TyCtxt<'_, '_, '_>, item: &ImplItem) -> bool {
     match item.node {
-        ImplItemKind::Method(_, eid) => is_relevant_expr(tcx, tcx.body_tables(eid), &tcx.hir.body(eid).value),
+        ImplItemKind::Method(_, eid) => is_relevant_expr(tcx, tcx.body_tables(eid), &tcx.hir().body(eid).value),
         _ => false,
     }
 }
@@ -250,7 +371,7 @@ fn is_relevant_trait(tcx: TyCtxt<'_, '_, '_>, item: &TraitItem) -> bool {
     match item.node {
         TraitItemKind::Method(_, TraitMethod::Required(_)) => true,
         TraitItemKind::Method(_, TraitMethod::Provided(eid)) => {
-            is_relevant_expr(tcx, tcx.body_tables(eid), &tcx.hir.body(eid).value)
+            is_relevant_expr(tcx, tcx.body_tables(eid), &tcx.hir().body(eid).value)
         },
         _ => false,
     }
@@ -258,9 +379,10 @@ fn is_relevant_trait(tcx: TyCtxt<'_, '_, '_>, item: &TraitItem) -> bool {
 
 fn is_relevant_block(tcx: TyCtxt<'_, '_, '_>, tables: &ty::TypeckTables<'_>, block: &Block) -> bool {
     if let Some(stmt) = block.stmts.first() {
-        match stmt.node {
-            StmtKind::Decl(_, _) => true,
-            StmtKind::Expr(ref expr, _) | StmtKind::Semi(ref expr, _) => is_relevant_expr(tcx, tables, expr),
+        match &stmt.node {
+            StmtKind::Local(_) => true,
+            StmtKind::Expr(expr) | StmtKind::Semi(expr) => is_relevant_expr(tcx, tables, expr),
+            _ => false,
         }
     } else {
         block.expr.as_ref().map_or(false, |e| is_relevant_expr(tcx, tables, e))
@@ -268,18 +390,20 @@ fn is_relevant_block(tcx: TyCtxt<'_, '_, '_>, tables: &ty::TypeckTables<'_>, blo
 }
 
 fn is_relevant_expr(tcx: TyCtxt<'_, '_, '_>, tables: &ty::TypeckTables<'_>, expr: &Expr) -> bool {
-    match expr.node {
-        ExprKind::Block(ref block, _) => is_relevant_block(tcx, tables, block),
-        ExprKind::Ret(Some(ref e)) => is_relevant_expr(tcx, tables, e),
+    match &expr.node {
+        ExprKind::Block(block, _) => is_relevant_block(tcx, tables, block),
+        ExprKind::Ret(Some(e)) => is_relevant_expr(tcx, tables, e),
         ExprKind::Ret(None) | ExprKind::Break(_, None) => false,
-        ExprKind::Call(ref path_expr, _) => if let ExprKind::Path(ref qpath) = path_expr.node {
-            if let Some(fun_id) = opt_def_id(tables.qpath_def(qpath, path_expr.hir_id)) {
-                !match_def_path(tcx, fun_id, &paths::BEGIN_PANIC)
+        ExprKind::Call(path_expr, _) => {
+            if let ExprKind::Path(qpath) = &path_expr.node {
+                if let Some(fun_id) = opt_def_id(tables.qpath_def(qpath, path_expr.hir_id)) {
+                    !match_def_path(tcx, fun_id, &paths::BEGIN_PANIC)
+                } else {
+                    true
+                }
             } else {
                 true
             }
-        } else {
-            true
         },
         _ => true,
     }
@@ -311,13 +435,14 @@ fn check_attrs(cx: &LateContext<'_, '_>, span: Span, name: Name, attrs: &[Attrib
                         cx,
                         EMPTY_LINE_AFTER_OUTER_ATTR,
                         begin_of_attr_to_item,
-                        "Found an empty line after an outer attribute. Perhaps you forgot to add a '!' to make it an inner attribute?"
+                        "Found an empty line after an outer attribute. \
+                         Perhaps you forgot to add a '!' to make it an inner attribute?",
                     );
                 }
             }
         }
 
-        if let Some(ref values) = attr.meta_item_list() {
+        if let Some(values) = attr.meta_item_list() {
             if values.len() != 1 || attr.name() != "inline" {
                 continue;
             }
@@ -337,7 +462,7 @@ fn check_attrs(cx: &LateContext<'_, '_>, span: Span, name: Name, attrs: &[Attrib
 }
 
 fn check_semver(cx: &LateContext<'_, '_>, span: Span, lit: &Lit) {
-    if let LitKind::Str(ref is, _) = lit.node {
+    if let LitKind::Str(is, _) = lit.node {
         if Version::parse(&is.as_str()).is_ok() {
             return;
         }
@@ -351,7 +476,7 @@ fn check_semver(cx: &LateContext<'_, '_>, span: Span, lit: &Lit) {
 }
 
 fn is_word(nmi: &NestedMetaItem, expected: &str) -> bool {
-    if let NestedMetaItemKind::MetaItem(ref mi) = nmi.node {
+    if let NestedMetaItemKind::MetaItem(mi) = &nmi.node {
         mi.is_word() && mi.name() == expected
     } else {
         false
@@ -371,3 +496,47 @@ fn is_present_in_source(cx: &LateContext<'_, '_>, span: Span) -> bool {
     }
     true
 }
+
+#[derive(Copy, Clone)]
+pub struct CfgAttrPass;
+
+impl LintPass for CfgAttrPass {
+    fn get_lints(&self) -> LintArray {
+        lint_array!(DEPRECATED_CFG_ATTR,)
+    }
+
+    fn name(&self) -> &'static str {
+        "DeprecatedCfgAttribute"
+    }
+}
+
+impl EarlyLintPass for CfgAttrPass {
+    fn check_attribute(&mut self, cx: &EarlyContext<'_>, attr: &Attribute) {
+        if_chain! {
+            // check cfg_attr
+            if attr.name() == "cfg_attr";
+            if let Some(items) = attr.meta_item_list();
+            if items.len() == 2;
+            // check for `rustfmt`
+            if let Some(feature_item) = items[0].meta_item();
+            if feature_item.name() == "rustfmt";
+            // check for `rustfmt_skip` and `rustfmt::skip`
+            if let Some(skip_item) = &items[1].meta_item();
+            if skip_item.name() == "rustfmt_skip" || skip_item.name() == "skip";
+            // Only lint outer attributes, because custom inner attributes are unstable
+            // Tracking issue: https://github.com/rust-lang/rust/issues/54726
+            if let AttrStyle::Outer = attr.style;
+            then {
+                span_lint_and_sugg(
+                    cx,
+                    DEPRECATED_CFG_ATTR,
+                    attr.span,
+                    "`cfg_attr` is deprecated for rustfmt and got replaced by tool_attributes",
+                    "use",
+                    "#[rustfmt::skip]".to_string(),
+                    Applicability::MachineApplicable,
+                );
+            }
+        }
+    }
+}
