@@ -1,7 +1,8 @@
-use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
-use crate::rustc::{declare_tool_lint, lint_array};
-use crate::rustc::hir::*;
 use crate::utils::span_lint;
+use rustc::hir::*;
+use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
+use rustc::{declare_tool_lint, lint_array};
+use rustc_data_structures::fx::FxHashSet;
 
 /// **What it does:** Checks for usage of blacklisted names for variables, such
 /// as `foo`.
@@ -23,14 +24,12 @@ declare_clippy_lint! {
 
 #[derive(Clone, Debug)]
 pub struct BlackListedName {
-    blacklist: Vec<String>,
+    blacklist: FxHashSet<String>,
 }
 
 impl BlackListedName {
-    pub fn new(blacklist: Vec<String>) -> Self {
-        Self {
-            blacklist,
-        }
+    pub fn new(blacklist: FxHashSet<String>) -> Self {
+        Self { blacklist }
     }
 }
 
@@ -38,12 +37,15 @@ impl LintPass for BlackListedName {
     fn get_lints(&self) -> LintArray {
         lint_array!(BLACKLISTED_NAME)
     }
+    fn name(&self) -> &'static str {
+        "BlacklistedName"
+    }
 }
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for BlackListedName {
     fn check_pat(&mut self, cx: &LateContext<'a, 'tcx>, pat: &'tcx Pat) {
-        if let PatKind::Binding(_, _, ident, _) = pat.node {
-            if self.blacklist.iter().any(|s| ident.name == *s) {
+        if let PatKind::Binding(.., ident, _) = pat.node {
+            if self.blacklist.contains(&ident.name.to_string()) {
                 span_lint(
                     cx,
                     BLACKLISTED_NAME,
