@@ -1,19 +1,22 @@
 #![allow(clippy::float_cmp)]
 
-use crate::rustc::lint::LateContext;
-use crate::rustc::{span_bug, bug};
-use crate::rustc::hir::def::Def;
-use crate::rustc::hir::*;
-use crate::rustc::ty::{self, Ty, TyCtxt, Instance};
-use crate::rustc::ty::subst::{Subst, Substs};
+use crate::utils::{clip, get_def_path, sext, unsext};
+use if_chain::if_chain;
+use rustc::hir::def::Def;
+use rustc::hir::*;
+use rustc::lint::LateContext;
+use rustc::ty::subst::{Subst, Substs};
+use rustc::ty::{self, Instance, Ty, TyCtxt};
+use rustc::{bug, span_bug};
+use rustc_data_structures::sync::Lrc;
 use std::cmp::Ordering::{self, Equal};
 use std::cmp::PartialOrd;
+use std::convert::TryFrom;
+use std::convert::TryInto;
 use std::hash::{Hash, Hasher};
-use std::mem;
-use std::rc::Rc;
-use crate::syntax::ast::{FloatTy, LitKind};
-use crate::syntax::ptr::P;
-use crate::utils::{sext, unsext, clip};
+use syntax::ast::{FloatTy, LitKind};
+use syntax::ptr::P;
+use syntax_pos::symbol::Symbol;
 
 /// A `LitKind`-like enum to fold constant `Expr`s into.
 #[derive(Debug, Clone)]
@@ -21,7 +24,7 @@ pub enum Constant {
     /// a String "abc"
     Str(String),
     /// a Binary String b"abc"
-    Binary(Rc<Vec<u8>>),
+    Binary(Lrc<Vec<u8>>),
     /// a single char 'a'
     Char(char),
     /// an integer's bit representation
@@ -38,6 +41,8 @@ pub enum Constant {
     Repeat(Box<Constant>, u64),
     /// a tuple of constants
     Tuple(Vec<Constant>),
+    /// a literal with syntax error
+    Err(Symbol),
 }
 
 impl PartialEq for Constant {
@@ -50,17 +55,19 @@ impl PartialEq for Constant {
             (&Constant::F64(l), &Constant::F64(r)) => {
                 // we want `Fw32 == FwAny` and `FwAny == Fw64`, by transitivity we must have
                 // `Fw32 == Fw64` so don’t compare them
-                // mem::transmute is required to catch non-matching 0.0, -0.0, and NaNs
-                unsafe { mem::transmute::<f64, u64>(l) == mem::transmute::<f64, u64>(r) }
+                // to_bits is required to catch non-matching 0.0, -0.0, and NaNs
+                l.to_bits() == r.to_bits()
             },
             (&Constant::F32(l), &Constant::F32(r)) => {
                 // we want `Fw32 == FwAny` and `FwAny == Fw64`, by transitivity we must have
                 // `Fw32 == Fw64` so don’t compare them
-                // mem::transmute is required to catch non-matching 0.0, -0.0, and NaNs
-                unsafe { mem::transmute::<f64, u64>(f64::from(l)) == mem::transmute::<f64, u64>(f64::from(r)) }
+                // to_bits is required to catch non-matching 0.0, -0.0, and NaNs
+                f64::from(l).to_bits() == f64::from(r).to_bits()
             },
             (&Constant::Bool(l), &Constant::Bool(r)) => l == r,
-            (&Constant::Vec(ref l), &Constant::Vec(ref r)) | (&Constant::Tuple(ref l), &Constant::Tuple(ref r)) => l == r,
+            (&Constant::Vec(ref l), &Constant::Vec(ref r)) | (&Constant::Tuple(ref l), &Constant::Tuple(ref r)) => {
+                l == r
+            },
             (&Constant::Repeat(ref lv, ref ls), &Constant::Repeat(ref rv, ref rs)) => ls == rs && lv == rv,
             _ => false, // TODO: Are there inter-type equalities?
         }
@@ -86,10 +93,10 @@ impl Hash for Constant {
                 i.hash(state);
             },
             Constant::F32(f) => {
-                unsafe { mem::transmute::<f64, u64>(f64::from(f)) }.hash(state);
+                f64::from(f).to_bits().hash(state);
             },
             Constant::F64(f) => {
-                unsafe { mem::transmute::<f64, u64>(f) }.hash(state);
+                f.to_bits().hash(state);
             },
             Constant::Bool(b) => {
                 b.hash(state);
@@ -101,17 +108,20 @@ impl Hash for Constant {
                 c.hash(state);
                 l.hash(state);
             },
+            Constant::Err(ref s) => {
+                s.hash(state);
+            },
         }
     }
 }
 
 impl Constant {
-    pub fn partial_cmp(tcx: TyCtxt<'_, '_, '_>, cmp_type: &ty::TyKind<'_>, left: &Self, right: &Self) -> Option<Ordering> {
+    pub fn partial_cmp(tcx: TyCtxt<'_, '_, '_>, cmp_type: ty::Ty<'_>, left: &Self, right: &Self) -> Option<Ordering> {
         match (left, right) {
             (&Constant::Str(ref ls), &Constant::Str(ref rs)) => Some(ls.cmp(rs)),
             (&Constant::Char(ref l), &Constant::Char(ref r)) => Some(l.cmp(r)),
             (&Constant::Int(l), &Constant::Int(r)) => {
-                if let ty::Int(int_ty) = *cmp_type {
+                if let ty::Int(int_ty) = cmp_type.sty {
                     Some(sext(tcx, l, int_ty).cmp(&sext(tcx, r, int_ty)))
                 } else {
                     Some(l.cmp(&r))
@@ -123,11 +133,11 @@ impl Constant {
             (&Constant::Tuple(ref l), &Constant::Tuple(ref r)) | (&Constant::Vec(ref l), &Constant::Vec(ref r)) => l
                 .iter()
                 .zip(r.iter())
-                .map(|(li, ri)| Constant::partial_cmp(tcx, cmp_type, li, ri))
+                .map(|(li, ri)| Self::partial_cmp(tcx, cmp_type, li, ri))
                 .find(|r| r.map_or(true, |o| o != Ordering::Equal))
                 .unwrap_or_else(|| Some(l.len().cmp(&r.len()))),
             (&Constant::Repeat(ref lv, ref ls), &Constant::Repeat(ref rv, ref rs)) => {
-                match Constant::partial_cmp(tcx, cmp_type, lv, rv) {
+                match Self::partial_cmp(tcx, cmp_type, lv, rv) {
                     Some(Equal) => Some(ls.cmp(rs)),
                     x => x,
                 }
@@ -139,25 +149,29 @@ impl Constant {
 
 /// parse a `LitKind` to a `Constant`
 pub fn lit_to_constant<'tcx>(lit: &LitKind, ty: Ty<'tcx>) -> Constant {
-    use crate::syntax::ast::*;
+    use syntax::ast::*;
 
     match *lit {
         LitKind::Str(ref is, _) => Constant::Str(is.to_string()),
         LitKind::Byte(b) => Constant::Int(u128::from(b)),
-        LitKind::ByteStr(ref s) => Constant::Binary(Rc::clone(s)),
+        LitKind::ByteStr(ref s) => Constant::Binary(Lrc::clone(s)),
         LitKind::Char(c) => Constant::Char(c),
         LitKind::Int(n, _) => Constant::Int(n),
-        LitKind::Float(ref is, _) |
-        LitKind::FloatUnsuffixed(ref is) => match ty.sty {
+        LitKind::Float(ref is, _) | LitKind::FloatUnsuffixed(ref is) => match ty.sty {
             ty::Float(FloatTy::F32) => Constant::F32(is.as_str().parse().unwrap()),
             ty::Float(FloatTy::F64) => Constant::F64(is.as_str().parse().unwrap()),
             _ => bug!(),
         },
         LitKind::Bool(b) => Constant::Bool(b),
+        LitKind::Err(s) => Constant::Err(s),
     }
 }
 
-pub fn constant<'c, 'cc>(lcx: &LateContext<'c, 'cc>, tables: &'c ty::TypeckTables<'cc>, e: &Expr) -> Option<(Constant, bool)> {
+pub fn constant<'c, 'cc>(
+    lcx: &LateContext<'c, 'cc>,
+    tables: &'c ty::TypeckTables<'cc>,
+    e: &Expr,
+) -> Option<(Constant, bool)> {
     let mut cx = ConstEvalLateContext {
         tcx: lcx.tcx,
         tables,
@@ -168,12 +182,19 @@ pub fn constant<'c, 'cc>(lcx: &LateContext<'c, 'cc>, tables: &'c ty::TypeckTable
     cx.expr(e).map(|cst| (cst, cx.needed_resolution))
 }
 
-pub fn constant_simple<'c, 'cc>(lcx: &LateContext<'c, 'cc>, tables: &'c ty::TypeckTables<'cc>, e: &Expr) -> Option<Constant> {
+pub fn constant_simple<'c, 'cc>(
+    lcx: &LateContext<'c, 'cc>,
+    tables: &'c ty::TypeckTables<'cc>,
+    e: &Expr,
+) -> Option<Constant> {
     constant(lcx, tables, e).and_then(|(cst, res)| if res { None } else { Some(cst) })
 }
 
 /// Creates a `ConstEvalLateContext` from the given `LateContext` and `TypeckTables`
-pub fn constant_context<'c, 'cc>(lcx: &LateContext<'c, 'cc>, tables: &'c ty::TypeckTables<'cc>) -> ConstEvalLateContext<'c, 'cc> {
+pub fn constant_context<'c, 'cc>(
+    lcx: &LateContext<'c, 'cc>,
+    tables: &'c ty::TypeckTables<'cc>,
+) -> ConstEvalLateContext<'c, 'cc> {
     ConstEvalLateContext {
         tcx: lcx.tcx,
         tables,
@@ -214,11 +235,37 @@ impl<'c, 'cc> ConstEvalLateContext<'c, 'cc> {
                 UnDeref => Some(o),
             }),
             ExprKind::Binary(op, ref left, ref right) => self.binop(op, left, right),
+            ExprKind::Call(ref callee, ref args) => {
+                // We only handle a few const functions for now
+                if_chain! {
+                    if args.is_empty();
+                    if let ExprKind::Path(qpath) = &callee.node;
+                    let def = self.tables.qpath_def(qpath, callee.hir_id);
+                    if let Some(def_id) = def.opt_def_id();
+                    let def_path = get_def_path(self.tcx, def_id);
+                    if let &["core", "num", impl_ty, "max_value"] = &def_path[..];
+                    then {
+                       let value = match impl_ty {
+                           "<impl i8>" => i8::max_value() as u128,
+                           "<impl i16>" => i16::max_value() as u128,
+                           "<impl i32>" => i32::max_value() as u128,
+                           "<impl i64>" => i64::max_value() as u128,
+                           "<impl i128>" => i128::max_value() as u128,
+                           _ => return None,
+                       };
+                       Some(Constant::Int(value))
+                    }
+                    else {
+                        None
+                    }
+                }
+            },
             // TODO: add other expressions
             _ => None,
         }
     }
 
+    #[allow(clippy::cast_possible_wrap)]
     fn constant_not(&self, o: &Constant, ty: ty::Ty<'_>) -> Option<Constant> {
         use self::Constant::*;
         match *o {
@@ -258,13 +305,13 @@ impl<'c, 'cc> ConstEvalLateContext<'c, 'cc> {
     /// create `Some(Vec![..])` of all constants, unless there is any
     /// non-constant part
     fn multi(&mut self, vec: &[Expr]) -> Option<Vec<Constant>> {
-        vec.iter()
-            .map(|elem| self.expr(elem))
-            .collect::<Option<_>>()
+        vec.iter().map(|elem| self.expr(elem)).collect::<Option<_>>()
     }
 
     /// lookup a possibly constant expression from a ExprKind::Path
     fn fetch_path(&mut self, qpath: &QPath, id: HirId) -> Option<Constant> {
+        use rustc::mir::interpret::GlobalId;
+
         let def = self.tables.qpath_def(qpath, id);
         match def {
             Def::Const(def_id) | Def::AssociatedConst(def_id) => {
@@ -279,9 +326,9 @@ impl<'c, 'cc> ConstEvalLateContext<'c, 'cc> {
                     instance,
                     promoted: None,
                 };
-                use crate::rustc::mir::interpret::GlobalId;
+
                 let result = self.tcx.const_eval(self.param_env.and(gid)).ok()?;
-                let ret = miri_to_const(self.tcx, result);
+                let ret = miri_to_const(self.tcx, &result);
                 if ret.is_some() {
                     self.needed_resolution = true;
                 }
@@ -317,55 +364,51 @@ impl<'c, 'cc> ConstEvalLateContext<'c, 'cc> {
         let l = self.expr(left)?;
         let r = self.expr(right);
         match (l, r) {
-            (Constant::Int(l), Some(Constant::Int(r))) => {
-                match self.tables.expr_ty(left).sty {
-                    ty::Int(ity) => {
-                        let l = sext(self.tcx, l, ity);
-                        let r = sext(self.tcx, r, ity);
-                        let zext = |n: i128| Constant::Int(unsext(self.tcx, n, ity));
-                        match op.node {
-                            BinOpKind::Add => l.checked_add(r).map(zext),
-                            BinOpKind::Sub => l.checked_sub(r).map(zext),
-                            BinOpKind::Mul => l.checked_mul(r).map(zext),
-                            BinOpKind::Div if r != 0 => l.checked_div(r).map(zext),
-                            BinOpKind::Rem if r != 0 => l.checked_rem(r).map(zext),
-                            BinOpKind::Shr => l.checked_shr(r as u128 as u32).map(zext),
-                            BinOpKind::Shl => l.checked_shl(r as u128 as u32).map(zext),
-                            BinOpKind::BitXor => Some(zext(l ^ r)),
-                            BinOpKind::BitOr => Some(zext(l | r)),
-                            BinOpKind::BitAnd => Some(zext(l & r)),
-                            BinOpKind::Eq => Some(Constant::Bool(l == r)),
-                            BinOpKind::Ne => Some(Constant::Bool(l != r)),
-                            BinOpKind::Lt => Some(Constant::Bool(l < r)),
-                            BinOpKind::Le => Some(Constant::Bool(l <= r)),
-                            BinOpKind::Ge => Some(Constant::Bool(l >= r)),
-                            BinOpKind::Gt => Some(Constant::Bool(l > r)),
-                            _ => None,
-                        }
+            (Constant::Int(l), Some(Constant::Int(r))) => match self.tables.expr_ty(left).sty {
+                ty::Int(ity) => {
+                    let l = sext(self.tcx, l, ity);
+                    let r = sext(self.tcx, r, ity);
+                    let zext = |n: i128| Constant::Int(unsext(self.tcx, n, ity));
+                    match op.node {
+                        BinOpKind::Add => l.checked_add(r).map(zext),
+                        BinOpKind::Sub => l.checked_sub(r).map(zext),
+                        BinOpKind::Mul => l.checked_mul(r).map(zext),
+                        BinOpKind::Div if r != 0 => l.checked_div(r).map(zext),
+                        BinOpKind::Rem if r != 0 => l.checked_rem(r).map(zext),
+                        BinOpKind::Shr => l.checked_shr(r.try_into().expect("invalid shift")).map(zext),
+                        BinOpKind::Shl => l.checked_shl(r.try_into().expect("invalid shift")).map(zext),
+                        BinOpKind::BitXor => Some(zext(l ^ r)),
+                        BinOpKind::BitOr => Some(zext(l | r)),
+                        BinOpKind::BitAnd => Some(zext(l & r)),
+                        BinOpKind::Eq => Some(Constant::Bool(l == r)),
+                        BinOpKind::Ne => Some(Constant::Bool(l != r)),
+                        BinOpKind::Lt => Some(Constant::Bool(l < r)),
+                        BinOpKind::Le => Some(Constant::Bool(l <= r)),
+                        BinOpKind::Ge => Some(Constant::Bool(l >= r)),
+                        BinOpKind::Gt => Some(Constant::Bool(l > r)),
+                        _ => None,
                     }
-                    ty::Uint(_) => {
-                        match op.node {
-                            BinOpKind::Add => l.checked_add(r).map(Constant::Int),
-                            BinOpKind::Sub => l.checked_sub(r).map(Constant::Int),
-                            BinOpKind::Mul => l.checked_mul(r).map(Constant::Int),
-                            BinOpKind::Div => l.checked_div(r).map(Constant::Int),
-                            BinOpKind::Rem => l.checked_rem(r).map(Constant::Int),
-                            BinOpKind::Shr => l.checked_shr(r as u32).map(Constant::Int),
-                            BinOpKind::Shl => l.checked_shl(r as u32).map(Constant::Int),
-                            BinOpKind::BitXor => Some(Constant::Int(l ^ r)),
-                            BinOpKind::BitOr => Some(Constant::Int(l | r)),
-                            BinOpKind::BitAnd => Some(Constant::Int(l & r)),
-                            BinOpKind::Eq => Some(Constant::Bool(l == r)),
-                            BinOpKind::Ne => Some(Constant::Bool(l != r)),
-                            BinOpKind::Lt => Some(Constant::Bool(l < r)),
-                            BinOpKind::Le => Some(Constant::Bool(l <= r)),
-                            BinOpKind::Ge => Some(Constant::Bool(l >= r)),
-                            BinOpKind::Gt => Some(Constant::Bool(l > r)),
-                            _ => None,
-                        }
-                    },
+                },
+                ty::Uint(_) => match op.node {
+                    BinOpKind::Add => l.checked_add(r).map(Constant::Int),
+                    BinOpKind::Sub => l.checked_sub(r).map(Constant::Int),
+                    BinOpKind::Mul => l.checked_mul(r).map(Constant::Int),
+                    BinOpKind::Div => l.checked_div(r).map(Constant::Int),
+                    BinOpKind::Rem => l.checked_rem(r).map(Constant::Int),
+                    BinOpKind::Shr => l.checked_shr(r.try_into().expect("shift too large")).map(Constant::Int),
+                    BinOpKind::Shl => l.checked_shl(r.try_into().expect("shift too large")).map(Constant::Int),
+                    BinOpKind::BitXor => Some(Constant::Int(l ^ r)),
+                    BinOpKind::BitOr => Some(Constant::Int(l | r)),
+                    BinOpKind::BitAnd => Some(Constant::Int(l & r)),
+                    BinOpKind::Eq => Some(Constant::Bool(l == r)),
+                    BinOpKind::Ne => Some(Constant::Bool(l != r)),
+                    BinOpKind::Lt => Some(Constant::Bool(l < r)),
+                    BinOpKind::Le => Some(Constant::Bool(l <= r)),
+                    BinOpKind::Ge => Some(Constant::Bool(l >= r)),
+                    BinOpKind::Gt => Some(Constant::Bool(l > r)),
                     _ => None,
-                }
+                },
+                _ => None,
             },
             (Constant::F32(l), Some(Constant::F32(r))) => match op.node {
                 BinOpKind::Add => Some(Constant::F32(l + r)),
@@ -398,7 +441,9 @@ impl<'c, 'cc> ConstEvalLateContext<'c, 'cc> {
             (l, r) => match (op.node, l, r) {
                 (BinOpKind::And, Constant::Bool(false), _) => Some(Constant::Bool(false)),
                 (BinOpKind::Or, Constant::Bool(true), _) => Some(Constant::Bool(true)),
-                (BinOpKind::And, Constant::Bool(true), Some(r)) | (BinOpKind::Or, Constant::Bool(false), Some(r)) => Some(r),
+                (BinOpKind::And, Constant::Bool(true), Some(r)) | (BinOpKind::Or, Constant::Bool(false), Some(r)) => {
+                    Some(r)
+                },
                 (BinOpKind::BitXor, Constant::Bool(l), Some(Constant::Bool(r))) => Some(Constant::Bool(l ^ r)),
                 (BinOpKind::BitAnd, Constant::Bool(l), Some(Constant::Bool(r))) => Some(Constant::Bool(l & r)),
                 (BinOpKind::BitOr, Constant::Bool(l), Some(Constant::Bool(r))) => Some(Constant::Bool(l | r)),
@@ -409,33 +454,34 @@ impl<'c, 'cc> ConstEvalLateContext<'c, 'cc> {
 }
 
 pub fn miri_to_const<'a, 'tcx>(tcx: TyCtxt<'a, 'tcx, 'tcx>, result: &ty::Const<'tcx>) -> Option<Constant> {
-    use crate::rustc::mir::interpret::{Scalar, ScalarMaybeUndef, ConstValue};
+    use rustc::mir::interpret::{ConstValue, Scalar};
     match result.val {
-        ConstValue::Scalar(Scalar::Bits{ bits: b, ..}) => match result.ty.sty {
+        ConstValue::Scalar(Scalar::Bits { bits: b, .. }) => match result.ty.sty {
             ty::Bool => Some(Constant::Bool(b == 1)),
             ty::Uint(_) | ty::Int(_) => Some(Constant::Int(b)),
-            ty::Float(FloatTy::F32) => Some(Constant::F32(f32::from_bits(b as u32))),
-            ty::Float(FloatTy::F64) => Some(Constant::F64(f64::from_bits(b as u64))),
+            ty::Float(FloatTy::F32) => Some(Constant::F32(f32::from_bits(
+                b.try_into().expect("invalid f32 bit representation"),
+            ))),
+            ty::Float(FloatTy::F64) => Some(Constant::F64(f64::from_bits(
+                b.try_into().expect("invalid f64 bit representation"),
+            ))),
             // FIXME: implement other conversion
             _ => None,
         },
-        ConstValue::ScalarPair(Scalar::Ptr(ptr),
-                               ScalarMaybeUndef::Scalar(
-                                Scalar::Bits { bits: n, .. })) => match result.ty.sty {
+        ConstValue::Slice(Scalar::Ptr(ptr), n) => match result.ty.sty {
             ty::Ref(_, tam, _) => match tam.sty {
                 ty::Str => {
-                    let alloc = tcx
-                        .alloc_map
-                        .lock()
-                        .unwrap_memory(ptr.alloc_id);
-                    let offset = ptr.offset.bytes() as usize;
-                    let n = n as usize;
-                    String::from_utf8(alloc.bytes[offset..(offset + n)].to_owned()).ok().map(Constant::Str)
+                    let alloc = tcx.alloc_map.lock().unwrap_memory(ptr.alloc_id);
+                    let offset = ptr.offset.bytes().try_into().expect("too-large pointer offset");
+                    let n = usize::try_from(n).unwrap();
+                    String::from_utf8(alloc.bytes[offset..(offset + n)].to_owned())
+                        .ok()
+                        .map(Constant::Str)
                 },
                 _ => None,
             },
             _ => None,
-        }
+        },
         // FIXME: implement other conversions
         _ => None,
     }
