@@ -1,9 +1,12 @@
-use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
-use crate::rustc::{declare_tool_lint, lint_array};
-use crate::rustc::hir::*;
-use crate::syntax::ast::NodeId;
-use crate::utils::{in_macro, match_def_path, match_trait_method, same_tys, snippet, span_lint_and_then};
+use crate::utils::{
+    in_macro, match_def_path, match_trait_method, same_tys, snippet, snippet_with_macro_callsite, span_lint_and_then,
+};
 use crate::utils::{opt_def_id, paths, resolve_node};
+use rustc::hir::*;
+use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
+use rustc::{declare_tool_lint, lint_array};
+use rustc_errors::Applicability;
+use syntax::ast::NodeId;
 
 /// **What it does:** Checks for always-identical `Into`/`From`/`IntoIter` conversions.
 ///
@@ -31,6 +34,10 @@ impl LintPass for IdentityConversion {
     fn get_lints(&self) -> LintArray {
         lint_array!(IDENTITY_CONVERSION)
     }
+
+    fn name(&self) -> &'static str {
+        "IdentityConversion"
+    }
 }
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for IdentityConversion {
@@ -61,9 +68,15 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for IdentityConversion {
                     let a = cx.tables.expr_ty(e);
                     let b = cx.tables.expr_ty(&args[0]);
                     if same_tys(cx, a, b) {
-                        let sugg = snippet(cx, args[0].span, "<expr>").into_owned();
+                        let sugg = snippet_with_macro_callsite(cx, args[0].span, "<expr>").to_string();
+
                         span_lint_and_then(cx, IDENTITY_CONVERSION, e.span, "identical conversion", |db| {
-                            db.span_suggestion(e.span, "consider removing `.into()`", sugg);
+                            db.span_suggestion(
+                                e.span,
+                                "consider removing `.into()`",
+                                sugg,
+                                Applicability::MachineApplicable, // snippet
+                            );
                         });
                     }
                 }
@@ -73,23 +86,36 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for IdentityConversion {
                     if same_tys(cx, a, b) {
                         let sugg = snippet(cx, args[0].span, "<expr>").into_owned();
                         span_lint_and_then(cx, IDENTITY_CONVERSION, e.span, "identical conversion", |db| {
-                            db.span_suggestion(e.span, "consider removing `.into_iter()`", sugg);
+                            db.span_suggestion(
+                                e.span,
+                                "consider removing `.into_iter()`",
+                                sugg,
+                                Applicability::MachineApplicable, // snippet
+                            );
                         });
                     }
                 }
             },
 
-            ExprKind::Call(ref path, ref args) => if let ExprKind::Path(ref qpath) = path.node {
-                if let Some(def_id) = opt_def_id(resolve_node(cx, qpath, path.hir_id)) {
-                    if match_def_path(cx.tcx, def_id, &paths::FROM_FROM[..]) {
-                        let a = cx.tables.expr_ty(e);
-                        let b = cx.tables.expr_ty(&args[0]);
-                        if same_tys(cx, a, b) {
-                            let sugg = snippet(cx, args[0].span.source_callsite(), "<expr>").into_owned();
-                            let sugg_msg = format!("consider removing `{}()`", snippet(cx, path.span, "From::from"));
-                            span_lint_and_then(cx, IDENTITY_CONVERSION, e.span, "identical conversion", |db| {
-                                db.span_suggestion(e.span, &sugg_msg, sugg);
-                            });
+            ExprKind::Call(ref path, ref args) => {
+                if let ExprKind::Path(ref qpath) = path.node {
+                    if let Some(def_id) = opt_def_id(resolve_node(cx, qpath, path.hir_id)) {
+                        if match_def_path(cx.tcx, def_id, &paths::FROM_FROM[..]) {
+                            let a = cx.tables.expr_ty(e);
+                            let b = cx.tables.expr_ty(&args[0]);
+                            if same_tys(cx, a, b) {
+                                let sugg = snippet(cx, args[0].span.source_callsite(), "<expr>").into_owned();
+                                let sugg_msg =
+                                    format!("consider removing `{}()`", snippet(cx, path.span, "From::from"));
+                                span_lint_and_then(cx, IDENTITY_CONVERSION, e.span, "identical conversion", |db| {
+                                    db.span_suggestion(
+                                        e.span,
+                                        &sugg_msg,
+                                        sugg,
+                                        Applicability::MachineApplicable, // snippet
+                                    );
+                                });
+                            }
                         }
                     }
                 }
