@@ -1,11 +1,12 @@
 //! checks for `#[inline]` on trait methods without bodies
 
-use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
-use crate::rustc::{declare_tool_lint, lint_array};
-use crate::rustc::hir::*;
-use crate::syntax::ast::{Attribute, Name};
 use crate::utils::span_lint_and_then;
 use crate::utils::sugg::DiagnosticBuilderExt;
+use rustc::hir::*;
+use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
+use rustc::{declare_tool_lint, lint_array};
+use rustc_errors::Applicability;
+use syntax::ast::{Attribute, Name};
 
 /// **What it does:** Checks for `#[inline]` on trait methods without bodies
 ///
@@ -34,6 +35,10 @@ impl LintPass for Pass {
     fn get_lints(&self) -> LintArray {
         lint_array!(INLINE_FN_WITHOUT_BODY)
     }
+
+    fn name(&self) -> &'static str {
+        "InlineFnWithoutBody"
+    }
 }
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Pass {
@@ -56,7 +61,7 @@ fn check_attrs(cx: &LateContext<'_, '_>, name: Name, attrs: &[Attribute]) {
             attr.span,
             &format!("use of `#[inline]` on trait method `{}` which has no body", name),
             |db| {
-                db.suggest_remove_item(cx, attr.span, "remove");
+                db.suggest_remove_item(cx, attr.span, "remove", Applicability::MachineApplicable);
             },
         );
     }
