@@ -2,17 +2,17 @@
 //!
 //! This lint is **deny** by default.
 
-use crate::rustc::lint::{LateContext, LateLintPass, Lint, LintArray, LintPass};
-use crate::rustc::{declare_tool_lint, lint_array};
-use crate::rustc::hir::*;
-use crate::rustc::hir::def::Def;
-use crate::rustc::ty::{self, TypeFlags};
-use crate::rustc::ty::adjustment::Adjust;
-use crate::rustc_errors::Applicability;
-use crate::rustc_typeck::hir_ty_to_ty;
-use crate::syntax_pos::{DUMMY_SP, Span};
-use std::ptr;
 use crate::utils::{in_constant, in_macro, is_copy, span_lint_and_then};
+use rustc::hir::def::Def;
+use rustc::hir::*;
+use rustc::lint::{LateContext, LateLintPass, Lint, LintArray, LintPass};
+use rustc::ty::adjustment::Adjust;
+use rustc::ty::{self, TypeFlags};
+use rustc::{declare_tool_lint, lint_array};
+use rustc_errors::Applicability;
+use rustc_typeck::hir_ty_to_ty;
+use std::ptr;
+use syntax_pos::{Span, DUMMY_SP};
 
 /// **What it does:** Checks for declaration of `const` items which is interior
 /// mutable (e.g. contains a `Cell`, `Mutex`, `AtomicXxxx` etc).
@@ -32,11 +32,11 @@ use crate::utils::{in_constant, in_macro, is_copy, span_lint_and_then};
 ///
 /// **Example:**
 /// ```rust
-/// use std::sync::atomic::{Ordering::SeqCst, AtomicUsize};
+/// use std::sync::atomic::{AtomicUsize, Ordering::SeqCst};
 ///
 /// // Bad.
 /// const CONST_ATOM: AtomicUsize = AtomicUsize::new(12);
-/// CONST_ATOM.store(6, SeqCst);             // the content of the atomic is unchanged
+/// CONST_ATOM.store(6, SeqCst); // the content of the atomic is unchanged
 /// assert_eq!(CONST_ATOM.load(SeqCst), 12); // because the CONST_ATOM in these lines are distinct
 ///
 /// // Good.
@@ -64,11 +64,11 @@ declare_clippy_lint! {
 ///
 /// **Example:**
 /// ```rust
-/// use std::sync::atomic::{Ordering::SeqCst, AtomicUsize};
+/// use std::sync::atomic::{AtomicUsize, Ordering::SeqCst};
 /// const CONST_ATOM: AtomicUsize = AtomicUsize::new(12);
 ///
 /// // Bad.
-/// CONST_ATOM.store(6, SeqCst);             // the content of the atomic is unchanged
+/// CONST_ATOM.store(6, SeqCst); // the content of the atomic is unchanged
 /// assert_eq!(CONST_ATOM.load(SeqCst), 12); // because the CONST_ATOM in these lines are distinct
 ///
 /// // Good.
@@ -84,16 +84,9 @@ declare_clippy_lint! {
 
 #[derive(Copy, Clone)]
 enum Source {
-    Item {
-        item: Span,
-    },
-    Assoc {
-        item: Span,
-        ty: Span,
-    },
-    Expr {
-        expr: Span,
-    },
+    Item { item: Span },
+    Assoc { item: Span, ty: Span },
+    Expr { expr: Span },
 }
 
 impl Source {
@@ -113,11 +106,7 @@ impl Source {
     }
 }
 
-fn verify_ty_bound<'a, 'tcx>(
-    cx: &LateContext<'a, 'tcx>,
-    ty: ty::Ty<'tcx>,
-    source: Source,
-) {
+fn verify_ty_bound<'a, 'tcx>(cx: &LateContext<'a, 'tcx>, ty: ty::Ty<'tcx>, source: Source) {
     if ty.is_freeze(cx.tcx, cx.param_env, DUMMY_SP) || is_copy(cx, ty) {
         // an UnsafeCell is !Copy, and an UnsafeCell is also the only type which
         // is !Freeze, thus if our type is Copy we can be sure it must be Freeze
@@ -133,34 +122,35 @@ fn verify_ty_bound<'a, 'tcx>(
         match source {
             Source::Item { .. } => {
                 let const_kw_span = span.from_inner_byte_pos(0, 5);
-                db.span_suggestion_with_applicability(
+                db.span_suggestion(
                     const_kw_span,
                     "make this a static item",
                     "static".to_string(),
                     Applicability::MachineApplicable,
                 );
-            }
+            },
             Source::Assoc { ty: ty_span, .. } => {
                 if ty.flags.contains(TypeFlags::HAS_FREE_LOCAL_NAMES) {
                     db.span_help(ty_span, &format!("consider requiring `{}` to be `Copy`", ty));
                 }
-            }
+            },
             Source::Expr { .. } => {
-                db.help(
-                    "assign this const to a local or static variable, and use the variable here",
-                );
-            }
+                db.help("assign this const to a local or static variable, and use the variable here");
+            },
         }
     });
 }
 
-
 pub struct NonCopyConst;
 
 impl LintPass for NonCopyConst {
     fn get_lints(&self) -> LintArray {
         lint_array!(DECLARE_INTERIOR_MUTABLE_CONST, BORROW_INTERIOR_MUTABLE_CONST)
     }
+
+    fn name(&self) -> &'static str {
+        "NonCopyConst"
+    }
 }
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for NonCopyConst {
@@ -174,18 +164,32 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for NonCopyConst {
     fn check_trait_item(&mut self, cx: &LateContext<'a, 'tcx>, trait_item: &'tcx TraitItem) {
         if let TraitItemKind::Const(hir_ty, ..) = &trait_item.node {
             let ty = hir_ty_to_ty(cx.tcx, hir_ty);
-            verify_ty_bound(cx, ty, Source::Assoc { ty: hir_ty.span, item: trait_item.span });
+            verify_ty_bound(
+                cx,
+                ty,
+                Source::Assoc {
+                    ty: hir_ty.span,
+                    item: trait_item.span,
+                },
+            );
         }
     }
 
     fn check_impl_item(&mut self, cx: &LateContext<'a, 'tcx>, impl_item: &'tcx ImplItem) {
         if let ImplItemKind::Const(hir_ty, ..) = &impl_item.node {
-            let item_node_id = cx.tcx.hir.get_parent_node(impl_item.id);
-            let item = cx.tcx.hir.expect_item(item_node_id);
+            let item_node_id = cx.tcx.hir().get_parent_node(impl_item.id);
+            let item = cx.tcx.hir().expect_item(item_node_id);
             // ensure the impl is an inherent impl.
             if let ItemKind::Impl(_, _, _, _, None, _, _) = item.node {
                 let ty = hir_ty_to_ty(cx.tcx, hir_ty);
-                verify_ty_bound(cx, ty, Source::Assoc { ty: hir_ty.span, item: impl_item.span });
+                verify_ty_bound(
+                    cx,
+                    ty,
+                    Source::Assoc {
+                        ty: hir_ty.span,
+                        item: impl_item.span,
+                    },
+                );
             }
         }
     }
@@ -208,34 +212,34 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for NonCopyConst {
             let mut dereferenced_expr = expr;
             let mut needs_check_adjustment = true;
             loop {
-                let parent_id = cx.tcx.hir.get_parent_node(cur_expr.id);
+                let parent_id = cx.tcx.hir().get_parent_node(cur_expr.id);
                 if parent_id == cur_expr.id {
                     break;
                 }
-                if let Some(Node::Expr(parent_expr)) = cx.tcx.hir.find(parent_id) {
+                if let Some(Node::Expr(parent_expr)) = cx.tcx.hir().find(parent_id) {
                     match &parent_expr.node {
                         ExprKind::AddrOf(..) => {
                             // `&e` => `e` must be referenced
                             needs_check_adjustment = false;
-                        }
+                        },
                         ExprKind::Field(..) => {
                             dereferenced_expr = parent_expr;
                             needs_check_adjustment = true;
-                        }
+                        },
                         ExprKind::Index(e, _) if ptr::eq(&**e, cur_expr) => {
                             // `e[i]` => desugared to `*Index::index(&e, i)`,
                             // meaning `e` must be referenced.
                             // no need to go further up since a method call is involved now.
                             needs_check_adjustment = false;
                             break;
-                        }
+                        },
                         ExprKind::Unary(UnDeref, _) => {
                             // `*e` => desugared to `*Deref::deref(&e)`,
                             // meaning `e` must be referenced.
                             // no need to go further up since a method call is involved now.
                             needs_check_adjustment = false;
                             break;
-                        }
+                        },
                         _ => break,
                     }
                     cur_expr = parent_expr;
