@@ -1,24 +1,14 @@
-// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
-use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
-use crate::rustc::{declare_tool_lint, lint_array};
-use if_chain::if_chain;
-use crate::rustc::hir::*;
-use crate::rustc::hir::def::Def;
 use crate::utils::sugg::Sugg;
-use crate::syntax::ptr::P;
+use if_chain::if_chain;
+use rustc::hir::def::Def;
+use rustc::hir::*;
+use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
+use rustc::{declare_tool_lint, lint_array};
+use syntax::ptr::P;
 
-use crate::utils::{match_def_path, match_type, span_lint_and_then};
 use crate::utils::paths::*;
-use crate::rustc_errors::Applicability;
+use crate::utils::{match_def_path, match_type, span_lint_and_then, SpanlessEq};
+use rustc_errors::Applicability;
 
 /// **What it does:** Checks for expressions that could be replaced by the question mark operator
 ///
@@ -38,7 +28,7 @@ use crate::rustc_errors::Applicability;
 /// ```rust
 /// option?;
 /// ```
-declare_clippy_lint!{
+declare_clippy_lint! {
     pub QUESTION_MARK,
     style,
     "checks for expressions that could be replaced by the question mark operator"
@@ -51,6 +41,10 @@ impl LintPass for Pass {
     fn get_lints(&self) -> LintArray {
         lint_array!(QUESTION_MARK)
     }
+
+    fn name(&self) -> &'static str {
+        "QuestionMark"
+    }
 }
 
 impl Pass {
@@ -65,34 +59,58 @@ impl Pass {
     /// If it matches, it will suggest to use the question mark operator instead
     fn check_is_none_and_early_return_none(cx: &LateContext<'_, '_>, expr: &Expr) {
         if_chain! {
-            if let ExprKind::If(ref if_expr, ref body, _) = expr.node;
-            if let ExprKind::MethodCall(ref segment, _, ref args) = if_expr.node;
+            if let ExprKind::If(if_expr, body, else_) = &expr.node;
+            if let ExprKind::MethodCall(segment, _, args) = &if_expr.node;
             if segment.ident.name == "is_none";
             if Self::expression_returns_none(cx, body);
             if let Some(subject) = args.get(0);
             if Self::is_option(cx, subject);
 
             then {
-                span_lint_and_then(
-                    cx,
-                    QUESTION_MARK,
-                    expr.span,
-                    "this block may be rewritten with the `?` operator",
-                    |db| {
-                        let receiver_str = &Sugg::hir(cx, subject, "..");
-
-                        db.span_suggestion_with_applicability(
-                            expr.span,
-                            "replace_it_with",
-                            format!("{}?;", receiver_str),
-                            Applicability::MachineApplicable, // snippet
-                        );
+                let receiver_str = &Sugg::hir(cx, subject, "..");
+                let mut replacement: Option<String> = None;
+                if let Some(else_) = else_ {
+                    if_chain! {
+                        if let ExprKind::Block(block, None) = &else_.node;
+                        if block.stmts.len() == 0;
+                        if let Some(block_expr) = &block.expr;
+                        if SpanlessEq::new(cx).ignore_fn().eq_expr(subject, block_expr);
+                        then {
+                            replacement = Some(format!("Some({}?)", receiver_str));
+                        }
                     }
-                )
+                } else if Self::moves_by_default(cx, subject) {
+                        replacement = Some(format!("{}.as_ref()?;", receiver_str));
+                } else {
+                        replacement = Some(format!("{}?;", receiver_str));
+                }
+
+                if let Some(replacement_str) = replacement {
+                    span_lint_and_then(
+                        cx,
+                        QUESTION_MARK,
+                        expr.span,
+                        "this block may be rewritten with the `?` operator",
+                        |db| {
+                            db.span_suggestion(
+                                expr.span,
+                                "replace_it_with",
+                                replacement_str,
+                                Applicability::MaybeIncorrect, // snippet
+                            );
+                        }
+                    )
+               }
             }
         }
     }
 
+    fn moves_by_default(cx: &LateContext<'_, '_>, expression: &Expr) -> bool {
+        let expr_ty = cx.tables.expr_ty(expression);
+
+        !expr_ty.is_copy_modulo_regions(cx.tcx, cx.param_env, expression.span)
+    }
+
     fn is_option(cx: &LateContext<'_, '_>, expression: &Expr) -> bool {
         let expr_ty = cx.tables.expr_ty(expression);
 
@@ -108,17 +126,15 @@ impl Pass {
 
                 false
             },
-            ExprKind::Ret(Some(ref expr)) => {
-                Self::expression_returns_none(cx, expr)
-            },
+            ExprKind::Ret(Some(ref expr)) => Self::expression_returns_none(cx, expr),
             ExprKind::Path(ref qp) => {
                 if let Def::VariantCtor(def_id, _) = cx.tables.qpath_def(qp, expression.hir_id) {
-                    return match_def_path(cx.tcx, def_id,  &OPTION_NONE);
+                    return match_def_path(cx.tcx, def_id, &OPTION_NONE);
                 }
 
                 false
             },
-            _ => false
+            _ => false,
         }
     }
 
@@ -127,7 +143,7 @@ impl Pass {
         if_chain! {
             if block.stmts.len() == 1;
             if let Some(expr) = block.stmts.iter().last();
-            if let StmtKind::Semi(ref expr, _) = expr.node;
+            if let StmtKind::Semi(ref expr) = expr.node;
             if let ExprKind::Ret(ref ret_expr) = expr.node;
             if let &Some(ref ret_expr) = ret_expr;
 
@@ -136,9 +152,13 @@ impl Pass {
             }
         }
 
-        // Check if the block has an implicit return expression
-        if let Some(ref ret_expr) = block.expr {
-            return Some(ret_expr.clone());
+        // Check for `return` without a semicolon.
+        if_chain! {
+            if block.stmts.len() == 0;
+            if let Some(ExprKind::Ret(Some(ret_expr))) = block.expr.as_ref().map(|e| &e.node);
+            then {
+                return Some(ret_expr.clone());
+            }
         }
 
         None
