@@ -1,20 +1,10 @@
-// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
-use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
-use crate::rustc::{declare_tool_lint, lint_array};
-use if_chain::if_chain;
-use crate::rustc::hir;
-use crate::rustc::hir::intravisit::{walk_expr, NestedVisitorMap, Visitor};
-use crate::syntax::ast;
 use crate::utils::{get_trait_def_id, span_lint};
+use if_chain::if_chain;
+use rustc::hir;
+use rustc::hir::intravisit::{walk_expr, NestedVisitorMap, Visitor};
+use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
+use rustc::{declare_tool_lint, lint_array};
+use syntax::ast;
 
 /// **What it does:** Lints for suspicious operations in impls of arithmetic operators, e.g.
 /// subtracting elements in an Add impl.
@@ -67,26 +57,29 @@ impl LintPass for SuspiciousImpl {
     fn get_lints(&self) -> LintArray {
         lint_array![SUSPICIOUS_ARITHMETIC_IMPL, SUSPICIOUS_OP_ASSIGN_IMPL]
     }
+
+    fn name(&self) -> &'static str {
+        "SuspiciousImpl"
+    }
 }
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for SuspiciousImpl {
     fn check_expr(&mut self, cx: &LateContext<'a, 'tcx>, expr: &'tcx hir::Expr) {
         if let hir::ExprKind::Binary(binop, _, _) = expr.node {
             match binop.node {
-                | hir::BinOpKind::Eq
+                hir::BinOpKind::Eq
                 | hir::BinOpKind::Lt
                 | hir::BinOpKind::Le
                 | hir::BinOpKind::Ne
                 | hir::BinOpKind::Ge
-                | hir::BinOpKind::Gt
-                => return,
+                | hir::BinOpKind::Gt => return,
                 _ => {},
             }
             // Check if the binary expression is part of another bi/unary expression
             // as a child node
-            let mut parent_expr = cx.tcx.hir.get_parent_node(expr.id);
+            let mut parent_expr = cx.tcx.hir().get_parent_node(expr.id);
             while parent_expr != ast::CRATE_NODE_ID {
-                if let hir::Node::Expr(e) = cx.tcx.hir.get(parent_expr) {
+                if let hir::Node::Expr(e) = cx.tcx.hir().get(parent_expr) {
                     match e.node {
                         hir::ExprKind::Binary(..)
                         | hir::ExprKind::Unary(hir::UnOp::UnNot, _)
@@ -94,12 +87,10 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for SuspiciousImpl {
                         _ => {},
                     }
                 }
-                parent_expr = cx.tcx.hir.get_parent_node(parent_expr);
+                parent_expr = cx.tcx.hir().get_parent_node(parent_expr);
             }
             // as a parent node
-            let mut visitor = BinaryExprVisitor {
-                in_binary_expr: false,
-            };
+            let mut visitor = BinaryExprVisitor { in_binary_expr: false };
             walk_expr(&mut visitor, expr);
 
             if visitor.in_binary_expr {
@@ -122,10 +113,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for SuspiciousImpl {
                     cx,
                     SUSPICIOUS_ARITHMETIC_IMPL,
                     binop.span,
-                    &format!(
-                        r#"Suspicious use of binary operator in `{}` impl"#,
-                        impl_trait
-                    ),
+                    &format!(r#"Suspicious use of binary operator in `{}` impl"#, impl_trait),
                 );
             }
 
@@ -162,10 +150,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for SuspiciousImpl {
                     cx,
                     SUSPICIOUS_OP_ASSIGN_IMPL,
                     binop.span,
-                    &format!(
-                        r#"Suspicious use of binary operator in `{}` impl"#,
-                        impl_trait
-                    ),
+                    &format!(r#"Suspicious use of binary operator in `{}` impl"#, impl_trait),
                 );
             }
         }
@@ -192,12 +177,12 @@ fn check_binop<'a>(
     }
 
     // Get the actually implemented trait
-    let parent_fn = cx.tcx.hir.get_parent(expr.id);
-    let parent_impl = cx.tcx.hir.get_parent(parent_fn);
+    let parent_fn = cx.tcx.hir().get_parent(expr.id);
+    let parent_impl = cx.tcx.hir().get_parent(parent_fn);
 
     if_chain! {
         if parent_impl != ast::CRATE_NODE_ID;
-        if let hir::Node::Item(item) = cx.tcx.hir.get(parent_impl);
+        if let hir::Node::Item(item) = cx.tcx.hir().get(parent_impl);
         if let hir::ItemKind::Impl(_, _, _, _, Some(ref trait_ref), _, _) = item.node;
         if let Some(idx) = trait_ids.iter().position(|&tid| tid == trait_ref.path.def.def_id());
         if binop != expected_ops[idx];
@@ -218,9 +203,7 @@ impl<'a, 'tcx: 'a> Visitor<'tcx> for BinaryExprVisitor {
         match expr.node {
             hir::ExprKind::Binary(..)
             | hir::ExprKind::Unary(hir::UnOp::UnNot, _)
-            | hir::ExprKind::Unary(hir::UnOp::UnNeg, _) => {
-                self.in_binary_expr = true
-            },
+            | hir::ExprKind::Unary(hir::UnOp::UnNeg, _) => self.in_binary_expr = true,
             _ => {},
         }
 
