@@ -1,19 +1,9 @@
-// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
-use crate::rustc::lint::{EarlyContext, EarlyLintPass, LintArray, LintPass};
-use crate::rustc::{declare_tool_lint, lint_array};
-use crate::syntax::ast::*;
-use crate::syntax::source_map::Span;
-use crate::syntax::symbol::LocalInternedString;
 use crate::utils::span_lint;
+use rustc::lint::{EarlyContext, EarlyLintPass, LintArray, LintPass};
+use rustc::{declare_tool_lint, lint_array};
+use syntax::ast::*;
+use syntax::source_map::Span;
+use syntax::symbol::LocalInternedString;
 
 /// **What it does:** Checks for imports that remove "unsafe" from an item's
 /// name.
@@ -42,6 +32,10 @@ impl LintPass for UnsafeNameRemoval {
     fn get_lints(&self) -> LintArray {
         lint_array!(UNSAFE_REMOVED_FROM_NAME)
     }
+
+    fn name(&self) -> &'static str {
+        "UnsafeNameRemoval"
+    }
 }
 
 impl EarlyLintPass for UnsafeNameRemoval {
@@ -62,14 +56,13 @@ fn check_use_tree(use_tree: &UseTree, cx: &EarlyContext<'_>, span: Span) {
                 .expect("use paths cannot be empty")
                 .ident;
             unsafe_to_safe_check(old_name, new_name, cx, span);
-        }
-        UseTreeKind::Simple(None, ..) |
-        UseTreeKind::Glob => {},
+        },
+        UseTreeKind::Simple(None, ..) | UseTreeKind::Glob => {},
         UseTreeKind::Nested(ref nested_use_tree) => {
             for &(ref use_tree, _) in nested_use_tree {
                 check_use_tree(use_tree, cx, span);
             }
-        }
+        },
     }
 }
 
@@ -81,7 +74,10 @@ fn unsafe_to_safe_check(old_name: Ident, new_name: Ident, cx: &EarlyContext<'_>,
             cx,
             UNSAFE_REMOVED_FROM_NAME,
             span,
-            &format!("removed \"unsafe\" from the name of `{}` in use as `{}`", old_str, new_str),
+            &format!(
+                "removed \"unsafe\" from the name of `{}` in use as `{}`",
+                old_str, new_str
+            ),
         );
     }
 }
