@@ -1,4 +1,3 @@
-#![feature(tool_lints)]
 #![allow(clippy::print_literal)]
 #![warn(clippy::useless_format)]
 
@@ -14,6 +13,7 @@ fn main() {
     format!("{}", "foo");
     format!("{:?}", "foo"); // don't warn about debug
     format!("{:8}", "foo");
+    format!("{:width$}", "foo", width = 8);
     format!("{:+}", "foo"); // warn when the format makes no difference
     format!("{:<}", "foo"); // warn when the format makes no difference
     format!("foo {}", "bar");
@@ -23,6 +23,7 @@ fn main() {
     format!("{}", arg);
     format!("{:?}", arg); // don't warn about debug
     format!("{:8}", arg);
+    format!("{:width$}", arg, width = 8);
     format!("{:+}", arg); // warn when the format makes no difference
     format!("{:<}", arg); // warn when the format makes no difference
     format!("foo {}", arg);
@@ -44,4 +45,14 @@ fn main() {
 
     // A format! inside a macro should not trigger a warning
     foo!("should not warn");
+
+    // precision on string means slicing without panicking on size:
+    format!("{:.1}", "foo"); // could be "foo"[..1]
+    format!("{:.10}", "foo"); // could not be "foo"[..10]
+    format!("{:.prec$}", "foo", prec = 1);
+    format!("{:.prec$}", "foo", prec = 10);
+
+    format!("{}", 42.to_string());
+    let x = std::path::PathBuf::from("/bar/foo/qux");
+    format!("{}", x.display().to_string());
 }
