@@ -1,6 +1,3 @@
-#![feature(tool_lints)]
-
-
 #![allow(dead_code)]
 
 extern crate core;
@@ -91,11 +88,11 @@ fn useless() {
 
         let _: *const usize = std::mem::transmute(5_isize);
 
-        let _  = 5_isize as *const usize;
+        let _ = 5_isize as *const usize;
 
-        let _: *const usize = std::mem::transmute(1+1usize);
+        let _: *const usize = std::mem::transmute(1 + 1usize);
 
-        let _  = (1+1_usize) as *const usize;
+        let _ = (1 + 1_usize) as *const usize;
     }
 }
 
@@ -191,9 +188,7 @@ fn transmute_ptr_to_ptr() {
     let s = "hello world".to_owned();
     let lp = LifetimeParam { s: &s };
     let _: &LifetimeParam<'static> = unsafe { std::mem::transmute(&lp) };
-    let _: &GenericParam<&LifetimeParam<'static>> = unsafe {
-        std::mem::transmute(&GenericParam { t: &lp})
-    };
+    let _: &GenericParam<&LifetimeParam<'static>> = unsafe { std::mem::transmute(&GenericParam { t: &lp }) };
 }
 
-fn main() { }
+fn main() {}
