@@ -1,15 +1,3 @@
-// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
-
-
 #![allow(clippy::write_literal)]
 #![warn(clippy::write_with_newline)]
 
@@ -38,4 +26,9 @@ fn main() {
     write!(&mut v, "Hello {} {}\n\n", "world", "#2");
     writeln!(&mut v, "\ndon't\nwarn\nfor\nmultiple\nnewlines\n"); // #3126
     writeln!(&mut v, "\nbla\n\n"); // #3126
+
+    // Escaping
+    write!(&mut v, "\\n"); // #3514
+    write!(&mut v, "\\\n");
+    write!(&mut v, "\\\\n");
 }
