@@ -1,19 +1,11 @@
-// Copyright 2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
+#![deny(rust_2018_idioms)]
 
 use std::env::args;
 use std::fs::read_dir;
 use std::path::Path;
 use std::process::{Command, exit};
 
-const FILES_TO_IGNORE: &[&str] = &["main.css"];
+const FILES_TO_IGNORE: &[&str] = &["light.css"];
 
 fn get_folders<P: AsRef<Path>>(folder_path: P) -> Vec<String> {
     let mut ret = Vec::with_capacity(10);
