@@ -8,19 +8,31 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-use config::config_type::ConfigType;
-use config::lists::*;
-use config::{Config, FileName};
-
-use isatty::stdout_isatty;
-
 use std::collections::HashSet;
 use std::path::{Path, PathBuf};
 
+use atty;
+
+use crate::config::config_type::ConfigType;
+use crate::config::lists::*;
+use crate::config::{Config, FileName};
+
+/// Macro that will stringify the enum variants or a provided textual repr
+#[macro_export]
+macro_rules! configuration_option_enum_stringify {
+    ($variant:ident) => {
+        stringify!($variant)
+    };
+
+    ($_variant:ident: $value:expr) => {
+        stringify!($value)
+    };
+}
+
 /// Macro for deriving implementations of Serialize/Deserialize for enums
 #[macro_export]
 macro_rules! impl_enum_serialize_and_deserialize {
-    ( $e:ident, $( $x:ident ),* ) => {
+    ( $e:ident, $( $variant:ident $(: $value:expr)* ),* ) => {
         impl ::serde::ser::Serialize for $e {
             fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
                 where S: ::serde::ser::Serializer
@@ -31,7 +43,9 @@ macro_rules! impl_enum_serialize_and_deserialize {
                 #[allow(unreachable_patterns)]
                 match *self {
                     $(
-                        $e::$x => serializer.serialize_str(stringify!($x)),
+                        $e::$variant => serializer.serialize_str(
+                            configuration_option_enum_stringify!($variant $(: $value)*)
+                        ),
                     )*
                     _ => {
                         Err(S::Error::custom(format!("Cannot serialize {:?}", self)))
@@ -50,7 +64,7 @@ macro_rules! impl_enum_serialize_and_deserialize {
                 impl<'de, T> Visitor<'de> for StringOnly<T>
                         where T: ::serde::Deserializer<'de> {
                     type Value = String;
-                    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+                    fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
                         formatter.write_str("string")
                     }
                     fn visit_str<E>(self, value: &str) -> Result<String, E> {
@@ -59,11 +73,13 @@ macro_rules! impl_enum_serialize_and_deserialize {
                 }
                 let s = d.deserialize_string(StringOnly::<D>(PhantomData))?;
                 $(
-                    if stringify!($x).eq_ignore_ascii_case(&s) {
-                      return Ok($e::$x);
+                    if configuration_option_enum_stringify!($variant $(: $value)*)
+                        .eq_ignore_ascii_case(&s) {
+                      return Ok($e::$variant);
                     }
                 )*
-                static ALLOWED: &'static[&str] = &[$(stringify!($x),)*];
+                static ALLOWED: &'static[&str] = &[
+                    $(configuration_option_enum_stringify!($variant $(: $value)*),)*];
                 Err(D::Error::unknown_variant(&s, ALLOWED))
             }
         }
@@ -73,8 +89,9 @@ macro_rules! impl_enum_serialize_and_deserialize {
 
             fn from_str(s: &str) -> Result<Self, Self::Err> {
                 $(
-                    if stringify!($x).eq_ignore_ascii_case(s) {
-                        return Ok($e::$x);
+                    if configuration_option_enum_stringify!($variant $(: $value)*)
+                        .eq_ignore_ascii_case(s) {
+                        return Ok($e::$variant);
                     }
                 )*
                 Err("Bad variant")
@@ -85,7 +102,9 @@ macro_rules! impl_enum_serialize_and_deserialize {
             fn doc_hint() -> String {
                 let mut variants = Vec::new();
                 $(
-                    variants.push(stringify!($x));
+                    variants.push(
+                        configuration_option_enum_stringify!($variant $(: $value)*)
+                    );
                 )*
                 format!("[{}]", variants.join("|"))
             }
@@ -93,15 +112,25 @@ macro_rules! impl_enum_serialize_and_deserialize {
     };
 }
 
-macro_rules! configuration_option_enum{
-    ($e:ident: $( $x:ident ),+ $(,)*) => {
-        #[derive(Copy, Clone, Eq, PartialEq, Debug)]
+macro_rules! configuration_option_enum {
+    ($e:ident: $( $name:ident $(: $value:expr)* ),+ $(,)*) => (
+        #[derive(Copy, Clone, Eq, PartialEq)]
         pub enum $e {
-            $( $x ),+
+            $( $name ),+
         }
 
-        impl_enum_serialize_and_deserialize!($e, $( $x ),+);
-    }
+        impl ::std::fmt::Debug for $e {
+            fn fmt(&self, f: &mut ::std::fmt::Formatter<'_>) -> ::std::fmt::Result {
+                f.write_str(match self {
+                    $(
+                        $e::$name => configuration_option_enum_stringify!($name $(: $value)*),
+                    )+
+                })
+            }
+        }
+
+        impl_enum_serialize_and_deserialize!($e, $( $name $(: $value)* ),+);
+    );
 }
 
 configuration_option_enum! { NewlineStyle:
@@ -140,7 +169,7 @@ impl NewlineStyle {
     /// If the style is set to `Auto` and `raw_input_text` contains no
     /// newlines, the `Native` style will be used.
     pub(crate) fn apply(self, formatted_text: &mut String, raw_input_text: &str) {
-        use NewlineStyle::*;
+        use crate::NewlineStyle::*;
         let mut style = self;
         if style == Auto {
             style = Self::auto_detect(raw_input_text);
@@ -262,13 +291,20 @@ configuration_option_enum! { Color:
     Auto,
 }
 
+configuration_option_enum! { Version:
+    // 1.x.y
+    One,
+    // 2.x.y
+    Two,
+}
+
 impl Color {
     /// Whether we should use a coloured terminal.
-    pub fn use_colored_tty(&self) -> bool {
+    pub fn use_colored_tty(self) -> bool {
         match self {
             Color::Always => true,
             Color::Never => false,
-            Color::Auto => stdout_isatty(),
+            Color::Auto => atty::is(atty::Stream::Stdout),
         }
     }
 }
@@ -287,6 +323,9 @@ pub struct WidthHeuristics {
     // Maximum width of the args of a function call before falling back
     // to vertical formatting.
     pub fn_call_width: usize,
+    // Maximum width of the args of a function-like attributes before falling
+    // back to vertical formatting.
+    pub attr_fn_like_width: usize,
     // Maximum width in the body of a struct lit before falling back to
     // vertical formatting.
     pub struct_lit_width: usize,
@@ -308,6 +347,7 @@ impl WidthHeuristics {
     pub fn null() -> WidthHeuristics {
         WidthHeuristics {
             fn_call_width: usize::max_value(),
+            attr_fn_like_width: usize::max_value(),
             struct_lit_width: 0,
             struct_variant_width: 0,
             array_width: usize::max_value(),
@@ -319,6 +359,7 @@ impl WidthHeuristics {
     pub fn set(max_width: usize) -> WidthHeuristics {
         WidthHeuristics {
             fn_call_width: max_width,
+            attr_fn_like_width: max_width,
             struct_lit_width: max_width,
             struct_variant_width: max_width,
             array_width: max_width,
@@ -339,6 +380,7 @@ impl WidthHeuristics {
         };
         WidthHeuristics {
             fn_call_width: (60.0 * max_width_ratio).round() as usize,
+            attr_fn_like_width: (70.0 * max_width_ratio).round() as usize,
             struct_lit_width: (18.0 * max_width_ratio).round() as usize,
             struct_variant_width: (35.0 * max_width_ratio).round() as usize,
             array_width: (60.0 * max_width_ratio).round() as usize,
@@ -379,7 +421,8 @@ impl IgnoreList {
                     path.push(s);
                     path
                 }
-            }).collect();
+            })
+            .collect();
     }
 
     fn skip_file_inner(&self, file: &Path) -> bool {
@@ -411,13 +454,19 @@ pub trait CliOptions {
 }
 
 /// The edition of the compiler (RFC 2052)
-configuration_option_enum!{ Edition:
-    Edition2015,
-    Edition2018,
+configuration_option_enum! { Edition:
+    Edition2015: 2015,
+    Edition2018: 2018,
+}
+
+impl Default for Edition {
+    fn default() -> Edition {
+        Edition::Edition2015
+    }
 }
 
 impl Edition {
-    pub(crate) fn to_libsyntax_pos_edition(&self) -> syntax_pos::edition::Edition {
+    pub(crate) fn to_libsyntax_pos_edition(self) -> syntax_pos::edition::Edition {
         match self {
             Edition::Edition2015 => syntax_pos::edition::Edition::Edition2015,
             Edition::Edition2018 => syntax_pos::edition::Edition::Edition2018,
