@@ -8,15 +8,15 @@ use std::time::{Duration, Instant};
 
 use syntax::ast;
 use syntax::errors::emitter::{ColorConfig, EmitterWriter};
-use syntax::errors::Handler;
+use syntax::errors::{DiagnosticBuilder, Handler};
 use syntax::parse::{self, ParseSess};
 use syntax::source_map::{FilePathMapping, SourceMap, Span};
 
-use comment::{CharClasses, FullCodeCharKind};
-use config::{Config, FileName, Verbosity};
-use issues::BadIssueSeeker;
-use visitor::{FmtVisitor, SnippetProvider};
-use {modules, source_file, ErrorKind, FormatReport, Input, Session};
+use crate::comment::{CharClasses, FullCodeCharKind};
+use crate::config::{Config, FileName, Verbosity};
+use crate::issues::BadIssueSeeker;
+use crate::visitor::{FmtVisitor, SnippetProvider};
+use crate::{modules, source_file, ErrorKind, FormatReport, Input, Session};
 
 // A map of the files of a crate, with their new content
 pub(crate) type SourceFile = Vec<FileRecord>;
@@ -73,7 +73,12 @@ fn format_project<T: FormatHandler>(
     let source_map = Rc::new(SourceMap::new(FilePathMapping::empty()));
     let mut parse_session = make_parse_sess(source_map.clone(), config);
     let mut report = FormatReport::new();
-    let krate = parse_crate(input, &parse_session, config, &mut report)?;
+    let krate = match parse_crate(input, &parse_session, config, &mut report) {
+        Ok(krate) => krate,
+        // Surface parse error via Session (errors are merged there from report)
+        Err(ErrorKind::ParseError) => return Ok(report),
+        Err(e) => return Err(e),
+    };
     timer = timer.done_parsing();
 
     // Suppress error output if we have to do any further parsing.
@@ -106,7 +111,7 @@ fn format_project<T: FormatHandler>(
 
 // Used for formatting files.
 #[derive(new)]
-struct FormatContext<'a, T: FormatHandler + 'a> {
+struct FormatContext<'a, T: FormatHandler> {
     krate: &'a ast::Crate,
     report: FormatReport,
     parse_session: ParseSess,
@@ -152,7 +157,7 @@ impl<'a, T: FormatHandler + 'a> FormatContext<'a, T> {
 
         debug_assert_eq!(
             visitor.line_number,
-            ::utils::count_newlines(&visitor.buffer)
+            crate::utils::count_newlines(&visitor.buffer)
         );
 
         // For some reason, the source_map does not include terminating
@@ -604,22 +609,33 @@ fn parse_crate(
 ) -> Result<ast::Crate, ErrorKind> {
     let input_is_stdin = input.is_text();
 
-    let mut parser = match input {
-        Input::File(file) => parse::new_parser_from_file(parse_session, &file),
-        Input::Text(text) => parse::new_parser_from_source_str(
+    let parser = match input {
+        Input::File(file) => Ok(parse::new_parser_from_file(parse_session, &file)),
+        Input::Text(text) => parse::maybe_new_parser_from_source_str(
             parse_session,
             syntax::source_map::FileName::Custom("stdin".to_owned()),
             text,
-        ),
+        )
+        .map_err(|diags| {
+            diags
+                .into_iter()
+                .map(|d| DiagnosticBuilder::new_diagnostic(&parse_session.span_diagnostic, d))
+                .collect::<Vec<_>>()
+        }),
     };
 
-    parser.cfg_mods = false;
-    if config.skip_children() {
-        parser.recurse_into_file_modules = false;
-    }
+    let result = match parser {
+        Ok(mut parser) => {
+            parser.cfg_mods = false;
+            if config.skip_children() {
+                parser.recurse_into_file_modules = false;
+            }
 
-    let mut parser = AssertUnwindSafe(parser);
-    let result = catch_unwind(move || parser.0.parse_crate_mod());
+            let mut parser = AssertUnwindSafe(parser);
+            catch_unwind(move || parser.0.parse_crate_mod().map_err(|d| vec![d]))
+        }
+        Err(db) => Ok(Err(db)),
+    };
 
     match result {
         Ok(Ok(c)) => {
@@ -627,7 +643,7 @@ fn parse_crate(
                 return Ok(c);
             }
         }
-        Ok(Err(mut e)) => e.emit(),
+        Ok(Err(mut diagnostics)) => diagnostics.iter_mut().for_each(|d| d.emit()),
         Err(_) => {
             // Note that if you see this message and want more information,
             // then run the `parse_crate_mod` function above without
