@@ -48,11 +48,7 @@ where
     }
 }
 
-impl<T> Foo for Bar<T>
-where
-    T: Baz,
-{
-}
+impl<T> Foo for Bar<T> where T: Baz {}
 
 impl<T> Foo for Bar<T>
 where
@@ -133,11 +129,7 @@ mod m {
         }
     }
 
-    impl<T> PartialEq for S<T>
-    where
-        T: PartialEq,
-    {
-    }
+    impl<T> PartialEq for S<T> where T: PartialEq {}
 }
 
 impl<BorrowType, K, V, NodeType, HandleType>
@@ -230,3 +222,15 @@ impl<'a, 'b, 'c> SomeThing<Something>
 {
     fn foo() {}
 }
+
+// #2746
+impl<'seq1, 'seq2, 'body, 'scope, Channel>
+    Adc12<
+        Dual,
+        MasterRunningDma<'seq1, 'body, 'scope, Channel>,
+        SlaveRunningDma<'seq2, 'body, 'scope>,
+    >
+where
+    Channel: DmaChannel,
+{
+}
